--
-- Gallery schema definition.
--
-- This script is both a definition of the Gallery schema, and a means for
-- creating the database in a test environment (e.g. desktop.)
--

-- Why do we have artificial id as primary key in media_item and tag tables instead of
-- composite primary key (object_id_low, object_id_hi)?
-- SQLite table presents B-tree with rowid as a key. So any query by rowid is twice as effective as query by index.
-- Joins also should have benefit using rowid.
-- And the single INTEGER PRIMARY KEY column is used automatically as rowid if exists.
-- http://www.sqlite.org/lang_createtable.html#rowid
--
--
-- Represents video and photos.
--
-- TODO: add a state column to track whether photo is uploaded etc.
-- (need to consider which resolution is uploaded.)
--
CREATE TABLE media_item (
    id INTEGER PRIMARY KEY,
    type INTEGER NOT NULL,
    object_id_low INTEGER NOT NULL,
    object_id_hi INTEGER NOT NULL,
    width INTEGER NOT NULL,
    height INTEGER NOT NULL,
    name TEXT,
    asset_hash INTEGER,
    caption TEXT,
    local_path TEXT,
    date_created DATE NOT NULL,
    date_modified DATE NOT NULL,
    date_added DATE NOT NULL,
    date_created_utc DATE NOT NULL,
    lat TEXT,
    lon TEXT,
    duration INTEGER,
    uploaded_id_low INTEGER,
    uploaded_id_hi INTEGER,
    orientation INTEGER DEFAULT 0,
    parent_media_id_low INTEGER,
    parent_media_id_hi INTEGER,
    group_type INTEGER DEFAULT 0,
    sync_state INTEGER DEFAULT 0,
    potentially_orphaned INTEGER NOT NULL DEFAULT 0,
    has_date INTEGER DEFAULT 1,
    time_stamp DATE NOT NULL,
    mime_type TEXT DEFAULT 'image/jpeg',
    event_id INTEGER DEFAULT 0,
    full_md5 TEXT,
    file_size INTEGER DEFAULT 0
);

CREATE UNIQUE INDEX unique_media_item_oid_index ON media_item (
    object_id_low,
    object_id_hi
);

CREATE INDEX date_added_media_item_index ON media_item (
    date_added DESC, object_id_low DESC
);

CREATE INDEX time_stamp_media_item_index ON media_item (
    time_stamp DESC, object_id_low DESC
);

CREATE INDEX potentially_orphaned_media_item_index ON media_item (
    potentially_orphaned
);

CREATE INDEX type_media_item_index ON media_item (
    type
);

CREATE INDEX md5_media_item_index ON media_item (
    full_md5
);

CREATE INDEX local_cut_dupe_search_index ON media_item (
    name, mime_type, local_path
);


CREATE INDEX local_dupe_search_index ON media_item (
    name, width, height, mime_type, duration
);

--
-- Tracks binary properties for each media item (is favorite? in camera roll? etc.)
--
CREATE TABLE media_property (
    media_item_id INTEGER,
    property TEXT NOT NULL,
    PRIMARY KEY (media_item_id, property),
    FOREIGN KEY (media_item_id) REFERENCES media_item(id) ON DELETE CASCADE
);

--
-- Tracks key/value pairs associated with each media item.
--
-- TODO: do we need to add a "type" field?
--
CREATE TABLE metadata (
    media_item_id INTEGER NOT NULL,
    key TEXT NOT NULL,
    value TEXT NOT NULL,
    PRIMARY KEY (media_item_id, key),
    FOREIGN KEY (media_item_id) REFERENCES media_item(id) ON DELETE CASCADE
);

--
-- Tracks frames of group photos like lenticular
--
CREATE TABLE frame (
    media_item_id INTEGER NOT NULL,
    frame_number INTEGER NOT NULL,
    local_path NOT NULL,
    PRIMARY KEY (media_item_id, frame_number),
    FOREIGN KEY (media_item_id) REFERENCES media_item(id) ON DELETE CASCADE
);


CREATE TABLE tag_sync_revision(
    tag_revision_id INTEGER
);

INSERT INTO tag_sync_revision (tag_revision_id) VALUES (0);

--
-- Tags represent collections.
--
-- TODO: finalize columns (pending Senna model review.)
-- TODO: create indices on date (pending Senna model review.)
--
CREATE TABLE tag (
    id INTEGER PRIMARY KEY,
    object_id_low INTEGER NOT NULL,
    object_id_hi INTEGER NOT NULL,
    type INTEGER NOT NULL,
    label TEXT NOT NULL,
    date_created DATE,
    sort_date DATE,
    has_sort_date INTEGER DEFAULT 0,
    cover_id_low INTEGER,
    cover_id_high INTEGER,
    local_path TEXT,
    tag_revision_id INTEGER,
    misc INTEGER DEFAULT 0,
    update_status INTEGER DEFAULT 0,
    storage INTEGER CHECK (storage > 0), -- storage should be set explicitly
    new_content INTEGER DEFAULT 0,
    auto_save INTEGER DEFAULT 0,
    tag_sync_version INTEGER DEFAULT 0,
    media_items_sync_version INTEGER DEFAULT 0,
    time_stamp Date,
    custom_cover_id_low INTEGER,
    custom_cover_id_high INTEGER,
    db_version INTEGER,
    valid INTEGER DEFAULT 1

--    -- Place columns
--    longitude DOUBLE,
--    latitude DOUBLE,

--    -- Event columns
--    start_date DATE,
--    end_date DATE
);

CREATE UNIQUE INDEX unique_tag_object_id_index ON tag (
    object_id_low,
    object_id_hi
);

CREATE INDEX storage_option_tag_index ON tag (
    storage
);

--
-- Defines membership in a collection.
--
CREATE TABLE tag_membership (
    media_item_id INTEGER,
    tag_id INTEGER,
    tag_media_revision_id INTEGER,
    PRIMARY KEY (media_item_id, tag_id),
    FOREIGN KEY(media_item_id) REFERENCES media_item(id) ON DELETE CASCADE,
    FOREIGN KEY(tag_id) REFERENCES tag(id) ON DELETE CASCADE
);

CREATE INDEX tag_membership_tag_id_index ON tag_membership (
    tag_id
);

--
-- Represents a file in multi-part upload progress
--
CREATE TABLE multipart_upload (
    file_path TEXT,
    object_id TEXT,
    storage_key TEXT,
    upload_id TEXT,
    last_modified INTEGER,
    PRIMARY KEY (file_path)
);

--
-- Represents a single chunk of a file in multi-part upload progress
--
CREATE TABLE multipart_upload_part (
    file_path TEXT,
    chunk_number INTEGER,
    chunk_start_position INTEGER,
    chunk_url TEXT,
    headers TEXT,
    parameters TEXT,
    etag TEXT,
    PRIMARY KEY (file_path, chunk_number),
    FOREIGN KEY(file_path) REFERENCES multipart_upload(file_path) ON DELETE CASCADE
);

--
-- Stores users sorting preferences within albums
--
CREATE TABLE sort_preference (
    tag_id TEXT,
    sort_order TEXT,
    PRIMARY KEY (tag_id)
);
