// A constant representing the combined model/view/projection matrix.
uniform mat4 u_MVPMatrix;
uniform mat4 u_TexMatrix;
uniform float u_Alpha;
// here's a brightness value that we pass in
uniform float u_Brightness;
uniform vec2 u_ShadowScaleMesh;
uniform vec2 u_ShadowScaleTexCoord;
uniform mediump vec2 u_ImageTexCoordBorderTopLeft;
uniform mediump vec2 u_ImageTexCoordBorderTopRight;
uniform mediump vec2 u_ImageTexCoordBorderBottomLeft;
uniform mediump vec2 u_ImageTexCoordBorderBottomRight;

// Per-vertex position information we will pass in.
attribute vec4 a_Position;
// Per-vertex color information we will pass in.
attribute vec4 a_Color;
// tex coordinate that we will pass in
attribute vec2 a_TexCoordinate0;
// tex coordinate reference from 0 to 1
attribute vec2 a_TexCoordinateRef;

// tex coordinate to pass to the pixel shader
varying vec2 v_TexCoordinate0;

// This will be passed into the fragment shader.
varying vec4 v_Color;     

varying vec2 v_TexCoordTopLeft;
varying vec2 v_TexCoordTopRight;
varying vec2 v_TexCoordBottomLeft;
varying vec2 v_TexCoordBottomRight;

// The entry point for our vertex shader.
void main()
{                          
    // Pass the color through to the fragment shader.
    // It will be interpolated across the triangle.
    v_Color.rgb = a_Color.rgb * u_Brightness;
    
    // now, can set a alpha color for transparency
    v_Color.a = u_Alpha;

    // transform our tex coordinates here
    vec4 texTransformed = u_TexMatrix * vec4(a_TexCoordinate0.x, a_TexCoordinate0.y, 0.0, 1.0);
    texTransformed.xy = texTransformed.xy * u_ShadowScaleTexCoord;
    v_TexCoordinate0 = texTransformed.xy;
    vec2 shadowSize = (u_ShadowScaleTexCoord - 1.0) / 2.0;
    v_TexCoordinate0 = v_TexCoordinate0 - shadowSize;

    vec2 texScale = 0.5 / shadowSize;
    v_TexCoordBottomLeft = (texTransformed.xy - u_ImageTexCoordBorderBottomLeft * u_ShadowScaleTexCoord) * texScale;

    /**
     * readable code before optimization:
     */
    /*v_TexCoordTopLeft = (texTransformed.xy - u_ImageTexCoordBorderTopLeft * u_ShadowScaleTexCoord) * texScale + vec2(0.0, 1.0);
    //vertical flip to use the bottom left corner texture
    v_TexCoordTopLeft.y = 1.0 - v_TexCoordTopLeft.y
    v_TexCoordBottomRight = (texTransformed.xy - u_ImageTexCoordBorderBottomRight * u_ShadowScaleTexCoord) * texScale;
    //horizontal flip to use the bottom left corner texture
    v_TexCoordBottomRight.x = 1.0 - v_TexCoordBottomRight.x
    v_TexCoordTopRight = (texTransformed.xy - u_ImageTexCoordBorderTopRight * u_ShadowScaleTexCoord) * texScale + vec2(1.0);
    //mirror flip to use the bottom left corner texture
    v_TexCoordTopRight = vec2(1.0) - v_TexCoordTopRight; */

    /**
     * code after optimization:
     */
    v_TexCoordTopLeft = (texTransformed.xy - u_ImageTexCoordBorderTopLeft * u_ShadowScaleTexCoord) * texScale;
    v_TexCoordTopLeft.y = -v_TexCoordTopLeft.y;
    v_TexCoordBottomRight = (texTransformed.xy - u_ImageTexCoordBorderBottomRight * u_ShadowScaleTexCoord) * texScale;
    v_TexCoordBottomRight.x = -v_TexCoordBottomRight.x;
    v_TexCoordTopRight = (u_ImageTexCoordBorderTopRight * u_ShadowScaleTexCoord - texTransformed.xy) * texScale;

    vec4 scaledPos = vec4(a_Position.xy * u_ShadowScaleMesh, 0.0, 1.0);

    // gl_Position is a special variable used to store the final position.
    // Multiply the vertex by the matrix to get the final point in
    gl_Position = u_MVPMatrix * scaledPos;
}         