// A constant representing the combined model/view/projection matrix.
uniform mat4 u_MVPMatrix;
uniform mat4 u_TexMatrix;
uniform float u_Alpha;
uniform float u_TexelWidth;
uniform float u_TexelHeight;
uniform mediump mat3 u_ConvKernel;

// Per-vertex position information we will pass in.
attribute vec4 a_Position;
// Per-vertex color information we will pass in.
attribute vec4 a_Color;
// tex coordinate that we will pass in
attribute vec2 a_TexCoordinate0;
// tex coordinate reference from 0 to 1
attribute vec2 a_TexCoordinateRef;

// tex coordinate to pass to the pixel shader
varying vec2 v_TexCoordinate0;
// this is the reference tex coordinate from 0 to 1
varying vec2 v_TexCoordinateRef;
// This will be passed into the fragment shader.
varying vec4 v_Color;

// here are the convolution matrix texture coords
// that we're going to pre-compute
// and let the hardware do the interpolation
varying vec2 v_TopLeft;
varying vec2 v_TopRight;
varying vec2 v_BotLeft;
varying vec2 v_BotRight;

// The entry point for our vertex shader.
void main()
{
    // Pass the color through to the fragment shader.
    // It will be interpolated across the triangle.
    v_Color.rgb = a_Color.rgb;

    // now, can set a alpha color for transparency
    v_Color.a = u_Alpha;

    // transform our tex coordinates here
    vec4 texCoordinate = vec4(a_TexCoordinate0.x, a_TexCoordinate0.y, 0.0, 1.0);
    vec4 texTransformed = u_TexMatrix * texCoordinate;
    v_TexCoordinate0 = texTransformed.xy;
    v_TexCoordinateRef = a_TexCoordinateRef;

    // setup the steps 1 texel offsets
    // these offsets are relative from the center pixel
    // we offset a normalized about based on the convolution kernel weights
    // this gives us an bilinear interpolated in between texel color so that we'll
    // just sample 4 samples in the fragment shader instead of 9.
    float percentX = u_ConvKernel[1][0] / (u_ConvKernel[1][0] + u_ConvKernel[1][1]);
    float percentY = u_ConvKernel[0][1] / (u_ConvKernel[0][1] + u_ConvKernel[1][1]);
    float offsetX = u_TexelWidth * percentX;
    float offsetY = u_TexelHeight * percentY;

    vec2 widthStep = vec2(offsetX, 0.0);
    vec2 heightStep = vec2(0.0, offsetY);
    vec2 widthHeightStep = vec2(offsetX, offsetY);
    vec2 widthNegHeightStep = vec2(offsetX, -offsetY);

    // now precompute the neighbouring pixels tex coordinates
    // into varyings. This is so that we can take advantage
    // of the hardware to interpolate and give us the actual
    // neighbouring fragments for free in the frag shader
    v_TopLeft = texTransformed.xy - widthHeightStep;
    v_TopRight = texTransformed.xy + widthNegHeightStep;
    v_BotLeft = texTransformed.xy - widthNegHeightStep;
    v_BotRight = texTransformed.xy + widthHeightStep;

    // gl_Position is a special variable used to store the final position.
    // Multiply the vertex by the matrix to get the final point in
    gl_Position = u_MVPMatrix * a_Position;
}